----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    16:12:22 08/23/2011 
-- Design Name: 
-- Module Name:    peakDetector - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Fhrt die Peaksuche im Houghraum durch. Leider nur seriell
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
--Libraries
library ieee;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library work;
use work.testbild.ALL;
use work.shape_data.ALL;
use work.referenceData.all; 

entity peakDetector is
	port(
		clk : in std_logic;
		reset : in std_logic;
		enable : in std_logic;
		hough_space : in hough_array;
		peak_found : out std_logic;
		found_y : out std_logic_vector(3 downto 0);
		found_x : out std_logic_vector(3 downto 0)
	);
end peakDetector;

architecture Behavioral of peakDetector is
--Signal list. For each output, two signals
signal sig_peak_found, sig_peak_found_next : std_logic;
signal sig_found_y, sig_found_y_next : unsigned(3 downto 0);
signal sig_found_x, sig_found_x_next : unsigned(3 downto 0);

signal line_counter, col_counter : unsigned(3 downto 0) ;
signal line_counter_next, col_counter_next : unsigned(3 downto 0) ;

begin

	--Clocked process
	clk_proc : process(clk, reset)
	begin
		if (reset = '1') then
			sig_found_y <= (others => '0');
			sig_found_x <= (others => '0');
			sig_peak_found <= '0';
			line_counter <= (others => '0');
			col_counter <= (others => '0');
		elsif (clk'event and clk = '1') then
			if (enable = '1') then
				sig_found_y <= sig_found_y_next;
				sig_found_x <= sig_found_x_next;
				sig_peak_found <= sig_peak_found_next;
				line_counter <= line_counter_next;
				col_counter <= col_counter_next;
			end if;
		end if;
	end process clk_proc;

	--Combinatoric process sensitiv on input ports and current output port signals
	comb_proc : process(hough_space, sig_found_y, sig_found_x, sig_peak_found, line_counter, col_counter)
	begin
		--Save current signals to next
		
		sig_found_y_next <= sig_found_y;
		sig_found_x_next <= sig_found_x;
		sig_peak_found_next <= sig_peak_found;
		line_counter_next <= line_counter;
		col_counter_next <= col_counter;

		if (unsigned(hough_space(to_integer(line_counter), to_integer(col_counter))) >= NUMBER_OF_SHAPE_PIXELS) then	--peak detection on
		--if (hough_space(to_integer(line_counter), to_integer(col_counter)) >= 1) then		--peak detection OFFF
			sig_peak_found_next <= '1';
			sig_found_y_next <= line_counter;
			sig_found_x_next <= col_counter;
		else
			sig_peak_found_next <= '0';
			sig_found_y_next <= sig_found_y;
			sig_found_x_next <= sig_found_x;
		end if;
	
		if (col_counter < (HOUGH_WIDTH - 1)) then
			col_counter_next <= col_counter + 1;
		else
			col_counter_next <= (others => '0');
			if(line_counter < (HOUGH_HEIGHT - 1)) then
				line_counter_next <= line_counter + 1;
			else
				line_counter_next <= (others => '0');
			end if;
			
		end if;
		
	end process comb_proc;

	--Outputs: (Current output signals to output ports. Must be outside of processes!):
	found_y <= std_logic_vector(sig_found_y);
	found_x <= std_logic_vector(sig_found_x);
	peak_found <= sig_peak_found;
	
 end Behavioral;
