

--Simuliert das Top-Modul, welches die Houghtransformation und die Peaksuche beinhaltet. 
--Kann auch zur Post-Synthese und PAR-Simulation genutzt werden. Dabei mssen allerdings die Typen des Ports 
--hough_space_out angepasst werden und auerdem die korrekten Simulationsbibliotheken inkludiert werden.  
--liest ein Bild aus testbild.vhd ein und gibt die Pixel taktweise an serialIn. Anschlieend wird die Peakdetection 
--aktiviert


  LIBRARY ieee;
  USE ieee.std_logic_1164.ALL;
  USE ieee.numeric_std.ALL;
  
  library work;
  use work.referenceData.ALL;
  use work.testbild.ALL;
 
--For PAR-Simulation:
--library SIMPRIM;
--use SIMPRIM.VCOMPONENTS.ALL;
--use SIMPRIM.VPACKAGE.ALL;

--For Post-Synthesis Simulation
--library UNISIM;
--use UNISIM.VCOMPONENTS.ALL;
--use UNISIM.VPKG.ALL;

  

  ENTITY tbTopHT IS
  END tbTopHT;

  ARCHITECTURE behavior OF tbTopHT IS 

  -- Component Declaration
        COMPONENT topHT
        Port ( 
			clk : in  STD_LOGIC;
         reset : in  STD_LOGIC;
			enable_ht : in  STD_LOGIC;
			enable_pd : in  STD_LOGIC;
			--hough_space_out : out hough_array;
			serialIn : in  STD_LOGIC;
			peak_found : out  STD_LOGIC;
			found_x : out std_logic_vector(3 downto 0);
			found_y : out std_logic_vector(3 downto 0)
		);
        END COMPONENT;

      signal clk : STD_LOGIC := '0';
		signal reset : STD_LOGIC := '0';
		signal serialIn : std_logic := '0';
		signal enable_ht : STD_LOGIC;
		signal enable_pd : STD_LOGIC;
		--signal hough_space_out_sig : hough_array;
		signal peak_found : STD_LOGIC;
		signal found_y : std_logic_vector(3 downto 0);
		signal found_x : std_logic_vector(3 downto 0);
		
		 constant clk_period : time := 10 ns;

  BEGIN

  -- Component Instantiation
		uut: topHT PORT MAP(
			clk => clk,
			reset => reset,
			enable_ht => enable_ht,
			enable_pd => enable_pd,
			--hough_space_out => hough_space_out_sig,
			serialIn => serialIn,
			peak_found => peak_found,
			found_x => found_x,
			found_y => found_y
		);

	 -- Clock process definitions
   clk_process :process
   begin
		clk <= '0';
		wait for clk_period/2;
		clk <= '1';
		wait for clk_period/2;
   end process;
		  
  
	stim_proc : PROCESS
	BEGIN
		reset <= '0';
		enable_ht <= '0';
		enable_pd <= '0';
		serialIn <= '0';
		wait for clk_period * 2;
		reset <= '1';
		wait for clk_period * 2;
		reset <= '0';
		wait for clk_period * 2;
		
		enable_ht <= '1';
				      
		for y in 0 to IMAGE_HEIGHT - 1 loop
			for x in 0 to IMAGE_WIDTH - 1 loop
				serialIn <= TEST_IMAGE(y,x);		--serialIn: std_logic, 
				wait for clk_period;
			end loop;
		end loop;
		
		serialIn <= '0';	--For the case if last pixel of testimage was '1'
		
		wait for clk_period;
		
		enable_ht <= '0';
		
		wait for clk_period;
		
		enable_pd <= '1';
		
		wait for clk_period * (HOUGH_HEIGHT * HOUGH_WIDTH);
		
		enable_pd <= '0';
		
		wait;		--for ever, i.e. terminate this process
		
	END PROCESS stim_proc;
	
	verify_proc : process
	variable error : std_logic := '0';
	begin
		
		wait for clk_period * 23;	--Wait all stimulus
		--wait for (clk_period * (IMAGE_HEIGHT * IMAGE_WIDTH));
		wait for (clk_period * (IMAGE_HEIGHT * IMAGE_WIDTH));
		
		for h in 0 to HOUGH_HEIGHT - 1 loop
			for w in 0 to HOUGH_WIDTH - 1 loop
			
			if (hough_space_out_sig(h,w) /= HOUGH_REFERENCE_DATA(h,w)) then
				error := '1';
				report "Error at y=" & INTEGER'image(h) & ", and x= " & INTEGER'image(w);
			end if;
			end loop;
		end loop;
		
		if(error = '1') then
			report "Verification failed";
		else
			report "Verification successful";
		end if;
		
		wait;
	
  
	end process verify_proc;
	
	
	
  --  End Test Bench 

  END;

  
  